7b6d3c70bd
* Moved tests/apps to tests/bin * Renamed cmake project in tests/bin * Update samples - Use ROCPROFILER_DEFAULT_FAIL_REGEX - tweaks to stdout messages * Update tests - Use ROCPROFILER_DEFAULT_FAIL_REGEX * Add tests/lib - libraries with HIP code * Update PTL submodule - remove atexit delete of thread_id_map * Update cmake/rocprofiler_options.cmake - Set ROCPROFILER_DEFAULT_FAIL_REGEX * Update common lib: env + logging - improved customization of logging settings - default to disabling logging to files - install failure handler for rocprofv3 - set_env support in environment.* * Add lib/rocprofiler-sdk/shared_library.cpp - shared library constructor * Update lib/rocprofiler-sdk-tool/tool.cpp - destructor thread safety - convert callback_name_info and buffered_name_info to pointers - install failure handler for logging * Add tests/bin/hip-in-libraries - hip-in-libraries is an exe which uses two shared libraries where each shared library contains HIP kernels - used for testing deadlocking within __hipRegisterFatBinary * Update bin/rocprofv3 - reorganized the env variables - use exec to launch command - set ROCPROFILER_LIBRARY_CTOR=1 * Add tests/rocprofv3/tracing-hip-in-libraries - uses hip-in-libraries exe for exe which uses shared libraries to launch HIP kernels * Update bin/rocprofv3 - fix counter collection (no exec) * Update lib/rocprofiler-sdk-tool/tool.cpp - replace "Kernel-Name" with "Kernel_Name" * Update lib/rocprofiler-sdk/registration.cpp Use RTLD_LOCAL instead of RTLD_GLOBAL for env libraries * Update tests/rocprofv3 - replace "Kernel-Name" with "Kernel_Name" * Update tests - vector-ops (bin) stream syncs + runs with 4 queues per device - improve counter-collection/input1 validation - rocprofv3/tracing-hip-in-libraries does not do sys-trace - improved validation script for tracing-hip-in-libraries - updated dispatch_callback in json-tool.cpp following reworking of prototypes for counter collection * Update samples/counter_collection - updated dispatch_callback(s) and record_callback(s) following reworking of prototypes * Update bin/rocprofv3 - reorganized help menu - added options for sub-HSA tables - added --hip-runtime-trace - changed --hip-trace to include --hip-compiler-trace * Update lib/rocprofiler-sdk-tool - improved kernel filtering - removed arch_vgpr, accum_vgpr, sgpr code (in rocprofiler-sdk) - fixed issue with counter-collection w/o tracing - added support for fine grained HSA API tracing - removed directly linking to HSA-runtime * Update lib/rocprofiler-sdk/agent.cpp - rocp_agents != hsa_agents is non-fatal when ROCPROFILER_BUILD_CI=OFF (CMake option) * GPR (vector and scalar) info in kernel symbol data - rocprofiler_callback_tracing_code_object_kernel_symbol_register_data_t contains general purpose register info * Header include order fix - Include repo headers first - Third party library headers next - standard library headers last * Update dispatch profiling public API - introduce rocprofiler_profile_counting_dispatch_data_t - change signature of rocprofiler_profile_counting_dispatch_callback_t and rocprofiler_profile_counting_record_callback_t - provide rocprofiler_user_data_t pointer in dispatch callback - provide rocprofiler_user_data_t value (from dispatch cb) in record callback * Update tests/bin/CMakeLists.txt - fix add_subdirectory(hip-in-libraries) order * Update VERSION - bump to 0.2.0 in prep for AFAR
152 líneas
5.1 KiB
C++
152 líneas
5.1 KiB
C++
// MIT License
|
|
//
|
|
// Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in
|
|
// all copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
// THE SOFTWARE.
|
|
|
|
#include "transpose.hpp"
|
|
#include "vector-ops.hpp"
|
|
|
|
#include <hip/hip_runtime_api.h>
|
|
|
|
#include <chrono>
|
|
#include <cstdio>
|
|
#include <cstdlib>
|
|
#include <iostream>
|
|
#include <mutex>
|
|
#include <stdexcept>
|
|
#include <thread>
|
|
|
|
#if defined(USE_MPI)
|
|
# include <mpi.h>
|
|
#endif
|
|
|
|
#define HIP_API_CALL(CALL) \
|
|
{ \
|
|
hipError_t error_ = (CALL); \
|
|
if(error_ != hipSuccess) \
|
|
{ \
|
|
auto _hip_api_print_lk = auto_lock_t{print_lock}; \
|
|
fprintf(stderr, \
|
|
"%s:%d :: HIP error : %s\n", \
|
|
__FILE__, \
|
|
__LINE__, \
|
|
hipGetErrorString(error_)); \
|
|
throw std::runtime_error("hip_api_call"); \
|
|
} \
|
|
}
|
|
|
|
namespace
|
|
{
|
|
using auto_lock_t = std::unique_lock<std::mutex>;
|
|
auto print_lock = std::mutex{};
|
|
size_t nqueues = 8;
|
|
size_t nthreads = 4;
|
|
size_t nitr = 500;
|
|
size_t nsync = 10;
|
|
} // namespace
|
|
|
|
int
|
|
main(int argc, char** argv)
|
|
{
|
|
int rank = 0;
|
|
int size = 1;
|
|
|
|
#if defined(USE_MPI)
|
|
MPI_Init(&argc, &argv);
|
|
MPI_Comm_rank(MPI_COMM_WORLD, &rank);
|
|
MPI_Comm_size(MPI_COMM_WORLD, &size);
|
|
#else
|
|
(void) size;
|
|
#endif
|
|
|
|
for(int i = 1; i < argc; ++i)
|
|
{
|
|
auto _arg = std::string{argv[i]};
|
|
if(_arg == "?" || _arg == "-h" || _arg == "--help")
|
|
{
|
|
if(rank == 0)
|
|
{
|
|
fprintf(stderr,
|
|
"usage: hip-in-libraries [NUM_QUEUES (%zu)] [NUM_THREADS (%zu)] "
|
|
"[NUM_ITERATION (%zu)] "
|
|
"[SYNC_EVERY_N_ITERATIONS (%zu)]\n",
|
|
nqueues,
|
|
nthreads,
|
|
nitr,
|
|
nsync);
|
|
}
|
|
exit(EXIT_SUCCESS);
|
|
}
|
|
}
|
|
|
|
if(argc > 1) nqueues = atoll(argv[1]);
|
|
if(argc > 2) nthreads = atoll(argv[2]);
|
|
if(argc > 3) nitr = atoll(argv[3]);
|
|
if(argc > 4) nsync = atoll(argv[4]);
|
|
|
|
int ndevice = 0;
|
|
HIP_API_CALL(hipGetDeviceCount(&ndevice));
|
|
|
|
printf("[hip-in-libraries] Number of devices found: %i\n", ndevice);
|
|
printf("[hip-in-libraries] Number of queues: %zu\n", nqueues);
|
|
printf("[hip-in-libraries] Number of threads: %zu\n", nthreads);
|
|
printf("[hip-in-libraries] Number of iterations: %zu\n", nitr);
|
|
printf("[hip-in-libraries] Syncing every %zu iterations\n", nsync);
|
|
|
|
{
|
|
auto vector_ops_thread = std::thread{run_vector_ops, nthreads, nqueues};
|
|
auto transpose_thread = std::thread{run_transpose, nthreads, nitr, nsync};
|
|
|
|
vector_ops_thread.join();
|
|
transpose_thread.join();
|
|
}
|
|
|
|
// this is a temporary workaround in omnitrace when HIP + MPI is enabled
|
|
|
|
#if defined(USE_MPI)
|
|
MPI_Barrier(MPI_COMM_WORLD);
|
|
#endif
|
|
|
|
for(int i = 0; i < ndevice; ++i)
|
|
{
|
|
HIP_API_CALL(hipSetDevice(i));
|
|
HIP_API_CALL(hipDeviceSynchronize());
|
|
}
|
|
|
|
#if defined(USE_MPI)
|
|
MPI_Barrier(MPI_COMM_WORLD);
|
|
#endif
|
|
|
|
if(rank == 0)
|
|
{
|
|
for(int i = 0; i < ndevice; ++i)
|
|
{
|
|
HIP_API_CALL(hipSetDevice(i));
|
|
HIP_API_CALL(hipDeviceReset());
|
|
}
|
|
}
|
|
|
|
#if defined(USE_MPI)
|
|
MPI_Barrier(MPI_COMM_WORLD);
|
|
#endif
|
|
|
|
return 0;
|
|
}
|